package Zonemaster::Backend;

our $VERSION = '6.1.0';

use strict;
use warnings;
use 5.14.2;

=head1 NAME

Zonemaster::Backend - A system for running Zonemaster tests asynchronously through an RPC-API

=head1 AUTHOR

Michal Toma <toma@nic.fr>

=head1 LICENSE

This is free software, licensed under:

The (three-clause) BSD License

The full text of the license can be found in the
F<LICENSE> file included with this distribution.

=cut

1;
